-- Libraries
local LibStub = LibStub
if not LibStub then
	error("PvPTimer requires LibStub.")
end

PvPTimer = LibStub("AceAddon-3.0"):NewAddon("PvPTimer", "AceConsole-3.0", "AceEvent-3.0")
local addon = PvPTimer
if not addon then
	error("PvPTimer requires Ace3.")
end

addon.Locale = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("PvPTimer")
local L = addon.Locale

addon.Lib = {}
addon.Lib.AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
addon.Lib.AceConfigDialog = LibStub("AceConfigDialog-3.0")
addon.Lib.AceGUI = LibStub("AceGUI-3.0")

addon.Lib.CallbackHandler = LibStub("CallbackHandler-1.0")
if not addon.Lib.CallbackHandler then
	error("PvPTimer requires CallbackHandler-1.0.")
end
addon.Lib.CallbackHandler:New(addon)

addon.Lib.Bars = LibStub("LibFancyBar-1.0")
if not addon.Lib.Bars then
	error("PvPTimer requires LibFancyBar-1.0.")
end

addon.Lib.Media = LibStub("LibSharedMedia-3.0")
if not addon.Lib.Media then
	error("PvPTimer requires LibSharedMedia-3.0.")
end

addon.Lib.Media:Register("sound", "Raid Warning", [[Sound\Interface\RaidWarning.wav]])

-- Constants
addon.Const = {}
addon.Spells = {}
addon.SpecSpells = {}

local tsort = table.sort
local pairs = pairs

local BT = LibStub("LibBabble-TalentTree-3.0"):GetUnstrictLookupTable()
local MSBT = MikSBT
local SCT = SCT
local SCTD = SCTD
local Parrot = Parrot
local IsAddonLoaded = IsAddonLoaded

CreateFrame("GameTooltip", "PT_Tooltip", UIParent, "GameTooltipTemplate")
PT_Tooltip:SetClampedToScreen(true)
PT_Tooltip:Hide()

L["MESSAGE_TAGS"] = L["MESSAGE_TAG1"]
		.."\n"..L["MESSAGE_TAG2"]
		.."\n"..L["MESSAGE_TAG10"]
		.."\n"..L["MESSAGE_TAG3"]
		.."\n"..L["MESSAGE_TAG4"]
		.."\n"..L["MESSAGE_TAG5"]
		.."\n"..L["MESSAGE_TAG6"]
		.."\n"..L["MESSAGE_TAG7"]
		.."\n"..L["MESSAGE_TAG8"]
		.."\n"..L["MESSAGE_TAG9"]

local msg1 = "%color_c%%player%||r "
local msg2 = " %icon%%color%%spell%"
local msg3 = " %icon_s%%color_s%%spec%"

-- default settings
addon.Defaults = {
	global = {
		build = 0,
		spells = {},
		anchor_default = {
			["Enabled"] = true,
			["Else"] = true,
			["Scale"] = 1.0,
			["BG"] = true,
			["RatedBG"] = true,
			["ShowSpec"] = 16,
			["SpecPosition"] = "RIGHT",
			["Width"] = 150,
			["Position"] = {
				["AttachFrame"] = "UIParent",
				["AttachRelative"] = "CENTER",
				["AttachX"] = "0",
				["AttachPoint"] = "CENTER",
				["AttachY"] = "-100",
			},
			["Bars"] = {
				["StartPoint"] = "bottomright",
				["IconMode"] = false,
				["IconSize"] = 32,
				["NameFont"] = "Arial Narrow",
				["Columns"] = 0,
				["MaxBars"] = 10,
				["Color"] = {
					["a"] = 0.7,
					["b"] = 0.5,
					["g"] = 0.3,
					["r"] = 0.2,
				},
				["TimeFont"] = "Arial Narrow",
				["TimeFontSize"] = 10,
				["GrowUp"] = false,
				["NameFontSize"] = 10,
				["ShowName"] = false,
				["ShowTime"] = true,
				["TypeColor"] = true,
				["Texture"] = "Blizzard",
				["Ghost"] = 5,
				["GhostBackground"] =  {
					["a"] = 0.8,
					["b"] = 0.8,
					["g"] = 0.1,
					["r"] = 0.1,
				},
				["GhostMessage"] = "|cFF00FF00Ready",
				["Fadeout"] = 1,
				["FadeoutBackground"] =  {
					["a"] = 0.8,
					["b"] = 0.8,
					["g"] = 0.1,
					["r"] = 0.1,
				},
				["FadeoutMessage"] = "|cFF00FF00Ready",
			},
			["Arena"] = true,
		},
	},
	profile = {
		Settings = {
			EnableAddon = true,
		},
		Spells = {},
		Anchors = {
			Anchor_Target = {
				Width = 150,
				Position = {
					["AttachFrame"] = "UIParent",
					["AttachRelative"] = "TOPLEFT",
					["AttachX"] = "200",
					["AttachPoint"] = "TOPLEFT",
					["AttachY"] = "-150",
				},
			},
			Anchor_Focus = {
				Width = 150,
				Position = {
					["AttachFrame"] = "UIParent",
					["AttachRelative"] = "TOPLEFT",
					["AttachX"] = "200",
					["AttachPoint"] = "TOPLEFT",
					["AttachY"] = "-320",
				},
			},
			Anchor_Arena = {
				Width = 120,
				Position = {
					["AttachFrame"] = "UIParent",
					["AttachRelative"] = "TOPLEFT",
					["AttachX"] = "270",
					["AttachPoint"] = "TOPLEFT",
					["AttachY"] = "-25",
					["NextX"] = 140,
					["NextY"] = 0,
				},
			},
		},
		Alert = {
			offensive = {
				Enabled = true,
				BG = "screen",
				Message = msg1..L["USED"]..msg2,
				BG_SoundEnable = false,
				BG_Sound = "Raid Warning",
				RatedBG_SoundEnable = false,
				RatedBG_Sound = "Raid Warning",
				Arena_SoundEnable = true,
				Arena_Sound = "Raid Warning",
				Else_SoundEnable = false,
				Else_Sound = "Raid Warning",
				RatedBG = "screen",
				Arena = "party",
				Else = "screen",
				BG_OnlyTarget = true,
				RatedBG_OnlyTarget = true,
				Arena_OnlyTarget = false,
				Else_OnlyTarget = true,
			},
			defensive = {
				Enabled = true,
				BG = "screen",
				Message = msg1..L["USED"]..msg2,
				BG_SoundEnable = false,
				BG_Sound = "Raid Warning",
				RatedBG_SoundEnable = false,
				RatedBG_Sound = "Raid Warning",
				Arena_SoundEnable = true,
				Arena_Sound = "Raid Warning",
				Else_SoundEnable = false,
				Else_Sound = "Raid Warning",
				RatedBG = "screen",
				Arena = "party",
				Else = "screen",
				BG_OnlyTarget = true,
				RatedBG_OnlyTarget = true,
				Arena_OnlyTarget = false,
				Else_OnlyTarget = true,
			},
			interrupt = {
				Enabled = true,
				OnlyTarget = false,
				BG = "screen",
				Message = msg1..L["USED"]..msg2,
				BG_SoundEnable = false,
				BG_Sound = "Raid Warning",
				RatedBG_SoundEnable = false,
				RatedBG_Sound = "Raid Warning",
				Arena_SoundEnable = true,
				Arena_Sound = "Raid Warning",
				Else_SoundEnable = false,
				Else_Sound = "Raid Warning",
				RatedBG = "screen",
				Arena = "party",
				Else = "screen",
				BG_OnlyTarget = true,
				RatedBG_OnlyTarget = true,
				Arena_OnlyTarget = false,
				Else_OnlyTarget = true,
			},
			cc = {
				Enabled = true,
				OnlyTarget = false,
				BG = "screen",
				Message = msg1..L["USED"]..msg2,
				BG_SoundEnable = false,
				BG_Sound = "Raid Warning",
				RatedBG_SoundEnable = false,
				RatedBG_Sound = "Raid Warning",
				Arena_SoundEnable = true,
				Arena_Sound = "Raid Warning",
				Else_SoundEnable = false,
				Else_Sound = "Raid Warning",
				RatedBG = "screen",
				Arena = "party",
				Else = "screen",
				BG_OnlyTarget = true,
				RatedBG_OnlyTarget = true,
				Arena_OnlyTarget = false,
				Else_OnlyTarget = true,
			},
			misc = {
				Enabled = true,
				OnlyTarget = false,
				BG = "default",
				Message = msg1..L["USED"]..msg2,
				BG_SoundEnable = false,
				BG_Sound = "Raid Warning",
				RatedBG_SoundEnable = false,
				RatedBG_Sound = "Raid Warning",
				Arena_SoundEnable = false,
				Arena_Sound = "Raid Warning",
				Else_SoundEnable = false,
				Else_Sound = "Raid Warning",
				RatedBG = "default",
				Arena = "default",
				Else = "default",
				BG_OnlyTarget = true,
				RatedBG_OnlyTarget = true,
				Arena_OnlyTarget = false,
				Else_OnlyTarget = true,
			},
			root = {
				Enabled = false,
				OnlyTarget = false,
				BG = "_none_",
				Message = msg1..L["USED"]..msg2,
				BG_SoundEnable = false,
				BG_Sound = "Raid Warning",
				RatedBG_SoundEnable = false,
				RatedBG_Sound = "Raid Warning",
				Arena_SoundEnable = false,
				Arena_Sound = "Raid Warning",
				Else_SoundEnable = false,
				Else_Sound = "Raid Warning",
				RatedBG = "_none_",
				Arena = "_none_",
				Else = "_none_",
				BG_OnlyTarget = true,
				RatedBG_OnlyTarget = true,
				Arena_OnlyTarget = false,
				Else_OnlyTarget = true,
			},
			spec = {
				Enabled = true,
				BG = "default",
				Message = msg1..L["SPECCED"]..msg3,
				BG_SoundEnable = false,
				BG_Sound = "Raid Warning",
				RatedBG_SoundEnable = false,
				RatedBG_Sound = "Raid Warning",
				Arena_SoundEnable = false,
				Arena_Sound = "Raid Warning",
				Else_SoundEnable = false,
				Else_Sound = "Raid Warning",
				RatedBG = "raid",
				Arena = "party",
				Else = "default",
				BG_OnlyTarget = false,
				RatedBG_OnlyTarget = false,
				Arena_OnlyTarget = false,
				Else_OnlyTarget = false,
			},
		},
		SpellCategories = {
			offensive = {
				["a"] = 0.7,
				["b"] = 0,
				["g"] = 0,
				["r"] = 1,
			},
			defensive = {
				["a"] = 0.7,
				["b"] = 0,
				["g"] = 1,
				["r"] = 0,
			},
			interrupt = {
				["a"] = 0.7,
				["b"] = 0,
				["g"] = 1,
				["r"] = 1,
			},
			cc = {
				["a"] = 0.7,
				["b"] = 1,
				["g"] = 0,
				["r"] = 1,
			},
			root = {
				["a"] = 0.7,
				["b"] = 0,
				["g"] = 0.5,
				["r"] = 1,
			},
			misc = {
				["a"] = 0.7,
				["b"] = 0.7,
				["g"] = 0.7,
				["r"] = 0.7,
			},
		},
		["Spells"] = {
			[586] = {
				["glyph3"] = true,
				["glyph0"] = true,
			},
			[47585] = {
				["glyph3"] = true,
			},
			[1850] = {
				["glyph3"] = true,
				["glyph0"] = true,
				["glyph1"] = true,
				["glyph2"] = true,
			},
			[86121] = {
				["glyph1"] = true,
			},
			[64044] = {
				["glyph3"] = true,
			},
		},
	},
}

addon.Const.On = "|cFF00FF00on"
addon.Const.Off = "|cFFFF0000off"
addon.Const.Racial = "|cFFa0a0a0Racial";
addon.Const.Item = "|cFFa0a0a0Item";

addon.Const.Classes = {}
FillLocalizedClassList(addon.Const.Classes)
tsort(addon.Const.Classes)

-- list of arena brackets
addon.Const.Arenas = {
	["2v2"] = "2v2",
	["3v3"] = "3v3",
	["5v5"] = "5v5",
}

-- list of targets for spell alerts
function addon:GetMessageFrames()
	-- builtin stuff
	local t = {
		["_none_"] = L["- DISABLED -"],
		["default"] = L["Default Chat Frame"],
		["screen"] = L["Faked RW"],
		["party"] = L["Party chat"],
		["raid"] = L["Raid chat"],
		["bg"] = L["Battleground chat"],
		["rw"] = L["Raid Warning"],
	}

	-- add MSBT frames if available
	if MSBT then
		for k, v in MSBT:IterateScrollAreas() do
			t["MSBT_"..k] = "MSBT: "..v
		end
	end

	-- add SCT frames if available
	if SCT then
		t["SCT_Frame1"] = "SCT: Frame 1"
		t["SCT_Frame2"] = "SCT: Frame 2"
		t["SCT_FrameMsg"] = "SCT: Message Frame"
		if SCTD then t["SCT_Damage"] = "SCT: Damage" end
	end

	-- add Parrot frames if available
	if Parrot then
		for k, v in pairs(Parrot:GetScrollAreasChoices()) do
			t["Parrot_"..k] = "Parrot: "..v
		end
	end

	if IsAddOnLoaded("Blizzard_CombatText") then
		t["BlizzCT"] = "Combat Text"
	end

	return t
end

-- frame attaching points
addon.Const.Attach = {
	["TOPLEFT"] = L["Top Left"],
	["TOP"] = L["Top"],
	["TOPRIGHT"] = L["Top Right"],
	["LEFT"] = L["Left"],
	["CENTER"] = L["Center"],
	["RIGHT"] = L["Right"],
	["BOTTOMLEFT"] = L["Bottom Left"],
	["BOTTOM"] = L["Bottom"],
	["BOTTOMRIGHT"] = L["Bottom Right"],
}

-- icon attaching points
addon.Const.IconPosition = {
	["INSIDELEFT"] = L["Above or below, on left"],
	["INSIDECENTER"] = L["Above or below, centered"],
	["INSIDERIGHT"] = L["Above or below, on right"],
	["LEFT"] = "Left",
	["RIGHT"] = "Right",
}

-- text justify options
addon.Const.Justify = {
	["LEFT"] = L["Left"],
	["CENTER"] = L["Center"],
	["RIGHT"] = L["Right"],
}

addon.Const.StartPoint = {
	["bottomright"] = L["Below on right"],
	["bottomleft"] = L["Below on left"],
	["topright"] = L["Above on right"],
	["topleft"] = L["Above on left"]
}

addon.Const.Units = {
	["Anchor_Target"] = "target",
	["Anchor_Focus"] = "focus",
	["Anchor_Arena1"] = "arena1",
	["Anchor_Arena2"] = "arena2",
	["Anchor_Arena3"] = "arena3",
	["Anchor_Arena4"] = "arena4",
	["Anchor_Arena5"] = "arena5",
}

--list of specs
addon.Const.Specs = {
	["default"] = {
		color = {r=1.0, g=1.0, b=1.0},
		icon = "Interface\\GossipFrame\\ActiveQuestIcon",
		name = L["Unknown"],
	},
	["DEATHKNIGHT"] = {
		[1] = {
			name = BT["Blood"],
			color = {r=1.0, g=0.0, b=0.0},
			icon = "Interface\\Icons\\spell_deathknight_bloodpresence",
		},
		[2] = {
			name = BT["Frost"],
			color = {r=0.3, g=0.5, b=1.0},
			icon = "Interface\\Icons\\spell_deathknight_frostpresence",
		},
		[3] = {
			name = BT["Unholy"],
			color = {r=0.2, g=0.8, b=0.2},
			icon = "Interface\\Icons\\spell_deathknight_unholypresence",
		}
	},

	["DRUID"] = {
		[1] = {
			name = BT["Balance"],
			color = {r=0.8, g=0.3, b=0.8},
			icon = "Interface\\Icons\\spell_nature_starfall",
		},
		[2] = {
			name = BT["Feral Combat"],
			color = {r=1.0, g=0.0, b=0.0},
			icon = "Interface\\Icons\\ability_racial_bearform",
		},
		[3] = {
			name = BT["Restoration"],
			color = {r=0.4, g=0.8, b=0.2},
			icon = "Interface\\Icons\\spell_nature_healingtouch",
		}
	},

	["HUNTER"] = {
		[1] = {
			name = BT["Beast Mastery"],
			color = {r=1.0, g=0.0, b=0.3},
			icon = "Interface\\Icons\\ability_hunter_bestialdiscipline",
		},
		[2] = {
			name = BT["Marksmanship"],
			color = {r=0.3, g=0.6, b=1.0},
			icon = "Interface\\Icons\\ability_hunter_focusedaim",
		},
		[3] = {
			name = BT["Survival"],
			color = {r=1.0, g=0.6, b=0.0},
			icon = "Interface\\Icons\\ability_hunter_camouflage",
		}
	},

	["MAGE"] = {
		[1] = {
			name = BT["Arcane"],
			color = {r=0.7, g=0.2, b=1.0},
			icon = "Interface\\Icons\\spell_holy_magicalsentry",
		},
		[2] = {
			name = BT["Fire"],
			color = {r=1.0, g=0.5, b=0.0},
			icon = "Interface\\Icons\\spell_fire_firebolt02",
		},
		[3] = {
			name = BT["Frost"],
			color = {r=0.3, g=0.6, b=1.0},
			icon = "Interface\\Icons\\spell_frost_frostbolt02",
		}
	},

	["PALADIN"] = {
		[1] = {
			name = BT["Holy"],
			color = {r=1.0, g=0.5, b=0.0},
			icon = "Interface\\Icons\\spell_holy_holybolt",
		},
		[2] = {
			name = BT["Protection"],
			color = {r=0.3, g=0.5, b=1.0},
			icon = "Interface\\Icons\\ability_paladin_shieldofthetemplar",
		},
		[3] = {
			name = BT["Retribution"],
			color = {r=1.0, g=0.0, b=0.0},
			icon = "Interface\\Icons\\spell_holy_auraoflight",
		}
	},

	["PRIEST"] = {
		[1] = {
			name = BT["Discipline"],
			color = {r=1.0, g=0.5, b=0.0},
			icon = "Interface\\Icons\\spell_holy_powerwordshield",
		},
		[2] = {
			name = BT["Holy"],
			color = {r=0.6, g=0.6, b=1.0},
			icon = "Interface\\Icons\\spell_holy_guardianspirit",
		},
		[3] = {
			name = BT["Shadow"],
			color = {r=0.7, g=0.4, b=0.8},
			icon = "Interface\\Icons\\spell_shadow_shadowwordpain",
		}
	},

	["ROGUE"] = {
		[1] = {
			name = BT["Assassination"],
			color = {r=0.5, g=0.8, b=0.5},
			icon = "Interface\\Icons\\ability_rogue_eviscerate",
		},
		[2] = {
			name = BT["Combat"],
			color = {r=1.0, g=0.5, b=0.0},
			icon = "Interface\\Icons\\ability_backstab",
		},
		[3] = {
			name = BT["Subtlety"],
			color = {r=0.3, g=0.5, b=1.0},
			icon = "Interface\\Icons\\ability_stealth",
		}
	},

	["SHAMAN"] = {
		[1] = {
			name = BT["Elemental"],
			color = {r=0.8, g=0.2, b=0.8},
			icon = "Interface\\Icons\\spell_nature_lightning",
		},
		[2] = {
			name = BT["Enhancement"],
			color = {r=0.3, g=0.5, b=1.0},
			icon = "Interface\\Icons\\spell_nature_lightningshield",
		},
		[3] = {
			name = BT["Restoration"],
			color = {r=0.2, g=0.8, b=0.4},
			icon = "Interface\\Icons\\spell_nature_magicimmunity",
		}
	},

	["WARLOCK"] = {
		[1] = {
			name = BT["Affliction"],
			color = {r=0.0, g=1.0, b=0.6},
			icon = "Interface\\Icons\\spell_shadow_deathcoil",
		},
		[2] = {
			name = BT["Demonology"],
			color = {r=1.0, g=0.0, b=0.0},
			icon = "Interface\\Icons\\spell_shadow_metamorphosis",
		},
		[3] = {
			name = BT["Destruction"],
			color = {r=1.0, g=0.5, b=0.0},
			icon = "Interface\\Icons\\spell_shadow_rainoffire",
		}
	},

	["WARRIOR"] = {
		[1] = {
			name = BT["Arms"],
			color = {r=1.0, g=0.72, b=0.1},
			icon = "Interface\\Icons\\ability_warrior_savageblow",
		},
		[2] = {
			name = BT["Fury"],
			color = {r=1.0, g=0.0, b=0.0},
			icon = "Interface\\Icons\\ability_warrior_innerrage",
		},
		[3] = {
			name = BT["Protection"],
			color = {r=0.3, g=0.5, b=1.0},
			icon = "Interface\\Icons\\ability_warrior_defensivestance",
		}
	},
}

addon.Const.DefaultIcon = addon.Const.Specs["default"].icon

